<?php
class Hook_SpamChecker
{
    public static function onBeforeClientSignUp(Box_Event $event)
    {
        self::_isSpam($event);
    }

    public static function onBeforeClientPublicTicketOpen(Box_Event $event)
    {
        self::_isSpam($event);
    }

    private static function _isSpam(Box_Event $event)
    {
        if(APPLICATION_ENV == 'testing') {
            return ;
        }

        $post   = $event->getSubject();
        $p      = $event->getParameters();
        
        $data = array(
            'ip'                        =>  isset($p['ip']) ? $p['ip'] : NULL,
            'email'                     =>  isset($post['email']) ? $post['email'] : NULL,
            'recaptcha_challenge_field' =>  isset($post['recaptcha_challenge_field']) ? $post['recaptcha_challenge_field'] : NULL,
            'recaptcha_response_field'  =>  isset($post['recaptcha_response_field']) ? $post['recaptcha_response_field'] : NULL,
        );
        
        self::_checkRecaptcha($data);
        self::_isInStopForumSpamDatabase($data);
    }
    
    private static function _checkRecaptcha(array $data)
    {
        $pdo = Box_Db::getPdo();
        
        $query = "SELECT value FROM setting WHERE param = :p";
        $stmt = $pdo->prepare($query);
        $stmt->execute(array('p'=>'captcha_enabled'));
        if(!$stmt->fetchColumn()) {
            return ;
        }
        
        if(!isset($data['ip']) ||
           !isset($data["recaptcha_challenge_field"]) ||
           !isset($data["recaptcha_response_field"]) ) {
            throw new Box_Exception('Captcha verification failed.');
        }

        $stmt->execute(array('p'=>'captcha_recaptcha_privatekey'));
        $privatekey = $stmt->fetchColumn();

        if ($privatekey == null || $privatekey == '') {
            throw new Box_Exception("To use reCAPTCHA you must get an API key from <a href='https://www.google.com/recaptcha/admin/create'>https://www.google.com/recaptcha/admin/create</a>");
        }

        require_once('recaptchalib.php');
        $resp = recaptcha_check_answer ($privatekey,
                                        $data['ip'],
                                        $data["recaptcha_challenge_field"],
                                        $data["recaptcha_response_field"]);
        if (!$resp->is_valid) {
            error_log($resp->error);
            throw new Box_Exception('Captcha verification failed.');
        }
    }
    
    /**
     * Pass params:
     *
     * ip
     * email
     * username
     *
     * @param array $data
     * @return bool
     */
    private static function _isInStopForumSpamDatabase(array $data)
    {
        $data['f'] = 'json';
        $url = 'http://www.stopforumspam.com/api?'.http_build_query($data);
        $file_contents = file_get_contents($url);

        $json = json_decode($file_contents);
        if(!is_object($json) || !$json->success) {
            return false;
        }

        if(isset($json->username->appears) && $json->username->appears) {
            throw new Box_Exception('Your Username is blacklisted in global database');
        }
        if(isset($json->email->appears) && $json->email->appears) {
            throw new Box_Exception('Your Email is blacklisted in global database');
        }
        if(isset($json->ip->appears) && $json->ip->appears) {
            throw new Box_Exception('Your IP is blacklisted in global database');
        }

        return false;
    }
}